@@ -2,7 +2,7 @@ import numpy as np
 from scipy.stats import chi2
 from pathlib import Path
 import re
-from nmrtoolbox.util import Range, ROI, AxisData, TableData, PeakOutsideOfMaskError, ParsePeakTableError
+from nmrtoolbox.util import Range, ROI, AxisData, TableData, PeakOutsideOfMaskError, ParsePeakTableError, is_iterable
 from nmrtoolbox.mask import Mask
 
 
@@ -10,18 +10,22 @@ class Peak:
     """
     A container for the values that describe a peak.
     """
-    def __init__(self, prop_list, val_list, axis_list):
+    def __init__(self, prop_list, val_list, axis_property):
         """
         Populate a peak
 
         :param prop_list: parameter names (from VARS line)
         :param val_list: parameter values (line from peak table)
-        :param axis_list: names of axes (e.g. 'X', 'Y', 'Z')
+        :param axis_property: AxisData object
         """
         if not (len(prop_list) == len(val_list)):
             raise ValueError('properties and values must be lists of the same length')
 
-        self.axis_list = axis_list
+        # this is an AxisData object
+        if isinstance(axis_property, AxisData):
+            self.axis_property = axis_property
+        else:
+            raise TypeError('axis_property must be AxisData type')
 
         # use the character in the format string to convert each value accordingly
         self.prop = dict()
@@ -39,13 +43,25 @@ class Peak:
                 pass
             self.prop[p] = v
 
-    def get_par(self, par, axis_list='all'):
+    def axis_keys(self):
+        """Get the axis keys"""
+        return self.axis_property.axis_keys()
+
+    def num_dims(self):
+        """Number of dimensions"""
+        return len(self.axis_keys())
+
+    def axis_labels(self):
+        """Get the axis labels"""
+        return self.axis_property.axis_labels()
+
+    def get_par(self, par, axis_keys='all'):
         """
         Get value of specified parameter for specified axes.
 
         :param par: nmrPipe peak table parameter name (e.g. *_PPM, HEIGHT)
                     The * is substituted with axis values for each label requested
-        :param axis_list: list of axis identifiers to include (e.g. 'X' or default of 'all')
+        :param axis_keys: list of axis identifiers to include (e.g. 'X' or default of 'all')
         :return: singleton value or list of values
         """
         if '*' not in par:
@@ -53,15 +69,15 @@ class Peak:
         else:
             par_set = []
 
-            # determine which labels to query
-            if axis_list == 'all':
-                axis_list = self.axis_list
-            if isinstance(axis_list, str):
-                axis_list = [axis_list]
+            # determine which axes to query
+            if axis_keys == 'all':
+                axis_keys = self.axis_property.axis_keys()
+            if isinstance(axis_keys, str):
+                axis_keys = [axis_keys]
 
             # perform substitutions of axis names into parameter name as needed
-            for axis in axis_list:
-                if axis not in self.axis_list:
+            for axis in axis_keys:
+                if axis not in self.axis_property.axis_keys():
                     raise KeyError(f'invalid axis: {axis}')
                 par_set.append(par.replace('*', axis))
 
@@ -102,21 +118,36 @@ class Peak:
         :param roi: ROI class object
         :return: bool
         """
-        if set(roi.axis_list()) != set(self.axis_list):
+        if not isinstance(roi, ROI):
+            raise TypeError('region of interest must be provided as ROI class object')
+
+        if roi.axis_labels() != self.axis_property.axis_labels():
             raise ValueError('the current peak and the given ROI do not have same set of axes')
 
-        return all([roi.range(axis).contains(self.get_par(f'{axis}_PPM')) for axis in self.axis_list])
+        for axis in self.axis_keys():
+            roi_range = roi.get_field(name='roi', axis=axis)[0]
+            peak_ppm = self.get_par(f'{axis}_PPM')
+            if not roi_range.contains(peak_ppm):
+                return False
+        return True
 
     def in_mask(self, mask: Mask, box_radius=2):
         """
         determine if the peak position (and some neighborhood around it) is in empty region of mask
         """
+        if not isinstance(mask, Mask):
+            raise TypeError('empty region of spectrum must be provided as Mask class object')
+
         # TODO:
-        #  the ordering of the dimensions in the Peak may not be consistent with ordering in Mask
-        #  this info would be available from PeakTable.axis_property
+        #  the Mask.axis_labels property is np.ndarray, so "==" is done pairwise and we need .all()
+        #  maybe upgrade Mask class to use axis_labels() method that returns a list?
+        if not (mask.axis_labels == self.axis_labels()).all():
+            raise ValueError('mask and peak have different axes')
+
         try:
             return mask.ppm_in_mask(
                 peak_ppm=self.get_par('*_PPM'),
+                peak_axis_labels=self.axis_labels(),
                 box_radius=box_radius,
             )
         except PeakOutsideOfMaskError as e:
@@ -168,10 +199,14 @@ class PeakTable:
         # list of Peak objects
         self.peaks = []
 
-        # metadata for each axis (e.g. X, Y, Z) - captured in an AxisData object:
+        # metadata for each axis (e.g. keys are X, Y, Z) - captured in an AxisData object:
         #   - labels (e.g.H1, N15)
         #   - num_pts
         #   - ppm_range (from ppm_high, ppm_low)
+        # TODO:
+        #  consider using axis labels as the keys instead of X, Y, Z
+        #  this would simplify how data objects are validated as compatible (currently looking
+        #  for 'label' in the AxisData, but could just be using the AxisData keys)
         self.axis_property = AxisData()
 
         # global properties that describe the collection of peaks
@@ -210,11 +245,16 @@ class PeakTable:
                 pass
 
     def _set_carrier_frequency(
-            self, carrier_frequency=None, carrier_frequency_unit='MHz', **kwargs):
-        # the injected peaks are presumed to be inserted into an empirical data set, so read the
-        # carrier frequency out of the empirical peak table, because it is not included in the
-        # injected peak table
-        # carrier_frequency = self.empirical_table.axis_property.get(name='carrier_frequency')
+            self, carrier_frequency=None, **kwargs):
+        """
+        Set the carrier frequency of the peak table.   Several types of input data are supported.
+        :param carrier_frequency:
+            If the input is omitted, then auto-determine CF from peak positions using ppm and hz values.
+            If the input is an AxisData object (e.g. from another PeakTable) - then extract the CF values.
+            Else - CF data can be given as a list of lists: [key, label, cf, cf_units]
+        :param kwargs:
+        :return:
+        """
 
         if carrier_frequency is None:
             # try to auto-determine carrier frequency from ppm and Hz values
@@ -224,32 +264,38 @@ class PeakTable:
                 raise KeyError(e)
 
         elif isinstance(carrier_frequency, AxisData):
-            if set(carrier_frequency.axis_list()) != set(self.axis_property.axis_list()):
+            if carrier_frequency.axis_labels() != self.axis_property.axis_labels():
                 raise AttributeError('The axes for the given carrier frequency data dot match those of the peak table')
-            self.axis_property.set_data(carrier_frequency)
+            # extract ONLY the carrier frequency values from the carrier_frequency AxisData object
+            # this object could contain other properties that we don't want
+            self.axis_property.set_data(carrier_frequency.get('carrier_frequency'))
 
         else:
-            # expect numerical data
-            try:
-                cf_values = np.asarray(carrier_frequency).astype(float, casting='safe')
-            except (ValueError, TypeError):
-                raise ValueError('unable to convert carrier frequency data')
+            msg = 'define carrier frequency by providing a list of lists: [key, label, cf, cf_units]'
+            if not is_iterable(carrier_frequency):
+                raise ValueError(msg)
 
-            if cf_values.size != self.num_dims():
-                raise AttributeError('number of carrier frequencies does not match number of peak table dimensions')
+            for info in carrier_frequency:
+                try:
+                    key, label, cf, cf_units = info
+                except ValueError:
+                    raise ValueError(msg)
 
-            if carrier_frequency_unit == 'Hz':
-                cf_hz = cf_values
-            elif carrier_frequency_unit == 'MHz':
-                cf_hz = cf_values * 1e6
-            else:
-                raise ValueError(f'do not recognize unit for carrier frequency: {carrier_frequency_unit}')
+                # confirm that the key/label combination provided as input match the existing axis_property data
+                if self.axis_property.get_field(axis=key, name='label')[0] != label:
+                    raise ValueError('carrier frequency data does not match axes labels')
+
+                if cf_units.lower() == 'hz':
+                    cf_hz = cf
+                elif cf_units.lower() == 'mhz':
+                    cf_hz = cf * 1e6
+                else:
+                    raise ValueError(f'do not recognize units for carrier frequency: {cf_units}.   must use "Hz" or "MHz"')
 
-            for ax, cf in zip(self.axis_list(), cf_hz):
                 self.axis_property.set(
-                    axis=ax,
+                    axis=key,
                     name='carrier_frequency',
-                    value=cf,
+                    value=cf_hz,
                     unit='Hz',
                 )
 
@@ -271,7 +317,7 @@ class PeakTable:
         except KeyError as e:
             raise KeyError(e)
 
-        for axis, cf in zip(self.axis_property.axis_list(), carrier):
+        for axis, cf in zip(self.axis_property.axis_keys(), carrier):
             self.axis_property.set(
                 axis=axis,
                 name='carrier_frequency',
@@ -282,7 +328,7 @@ class PeakTable:
     def _set_peak_positions_hz(self):
         """Use ppm peak positions and provided carrier frequencies to determine peak positions in Hz"""
 
-        for axis in self.axis_list():
+        for axis in self.axis_keys():
             try:
                 peak_ppm = self.get_par(par=f'{axis}_PPM')
             except KeyError:
@@ -290,9 +336,9 @@ class PeakTable:
             try:
                 unit = self.axis_property.get_field(name='carrier_frequency', axis=axis, field='unit')[0]
                 value = self.axis_property.get_field(name='carrier_frequency', axis=axis, field='value')[0]
-                if unit == 'Hz':
+                if unit.lower() == 'hz':
                     cf_mhz = value / 1e6
-                elif unit == 'MHz':
+                elif unit.lower() == 'mhz':
                     cf_mhz = value
                 else:
                     raise ValueError(f'do not recognize unit for carrier frequency: {unit}')
@@ -322,20 +368,20 @@ class PeakTable:
         except IndexError:
             raise IndexError('asking for index = {:d}, max index = {:d}'.format(index, len(self.peaks)))
 
-    def get_par(self, par, axis_list='all'):
+    def get_par(self, par, axis_keys='all'):
         """
         Get all values for specified parameter for all peaks for specified axes.
 
         :param par: nmrPipe peak table parameter name (e.g. *_PPM, HEIGHT)
                     The * is substituted with axis values for each label requested
-        :param axis_list: list of axis identifiers to include (e.g. 'X' or default of 'all')
+        :param axis_keys: list of axis identifiers to include (e.g. 'X' or default of 'all')
         :return: numpy array
         """
         out = []
         for peak in self.peaks:
             out.append(peak.get_par(
                 par=par,
-                axis_list=axis_list,
+                axis_keys=axis_keys,
             ))
         return np.array(out)
 
@@ -417,21 +463,21 @@ class PeakTable:
         """Reorder the peak list by HEIGHT (positive down to negative)"""
         self.peaks = sorted(self.peaks, key=lambda x: x.prop['HEIGHT'], reverse=True)
 
-    def axis_list(self):
-        """Get the axis names"""
-        return list(self.axis_property.axis_list())
+    def axis_keys(self):
+        """Get the axis keys"""
+        return self.axis_property.axis_keys()
 
     def num_dims(self):
         """Number of dimensions"""
-        return len(self.axis_list())
+        return len(self.axis_keys())
 
-    def label_list(self):
+    def axis_labels(self):
         """Get the axis labels"""
         return self.axis_property.get_field(name='label')
 
-    def axis_label_list(self):
+    def axis_keys_labels(self):
         """Get a list of tuples where each contains an axis and its label"""
-        return [(ax, lab) for ax, lab in zip(self.axis_list(), self.label_list())]
+        return [(ax, lab) for ax, lab in zip(self.axis_keys(), self.axis_labels())]
 
     def determine_outliers(self, chi2prob):
         # get full width of each recovered peak in points along each of its dimensions
@@ -578,7 +624,7 @@ class PeakTablePipe(PeakTable):
                         self.peaks.append(Peak(
                             prop_list=self.VARS,
                             val_list=line.split(),
-                            axis_list=self.axis_list(),
+                            axis_property=self.axis_property,
                         ))
 
 
@@ -606,7 +652,7 @@ class PeakTablePipeRec(PeakTablePipe):
         except KeyError as e:
             raise KeyError(e)
 
-        for axis, LW in zip(self.axis_property.axis_list(), LW_list):
+        for axis, LW in zip(self.axis_property.axis_keys(), LW_list):
             self.axis_property.set(
                 axis=axis,
                 name='LW',
@@ -667,7 +713,7 @@ class PeakTablePipeInj(PeakTablePipe):
     def _set_maxLW(self):
         # parse through the tokens to find maxLW values for each axis
         tokens = self.genSimTab.split()
-        for axis in self.axis_list():
+        for axis in self.axis_keys():
             try:
                 idx = tokens.index(f'-{axis.lower()}wMax')
                 self.axis_property.set(
