@@ -79,7 +79,7 @@ class roc:
             try:
                 self.injected_peaks = PeakTablePipeInj(
                     file=injected_table,
-                    carrier_frequency=self.recovered_peaks.axis_property.get('carrier_frequency'))
+                    carrier_frequency=self.recovered_peaks.axis_property)
             except FileNotFoundError as e:
                 raise FileNotFoundError(e)
             except ParsePeakTableError as e:
@@ -92,8 +92,8 @@ class roc:
         # ================================================
         self.roi = None
         if roi_list is not None:
-            axis_list = self.injected_peaks.axis_list()
-            if len(roi_list) != 2*len(axis_list):
+            axis_labels = self.injected_peaks.axis_labels()
+            if len(roi_list) != 2*len(axis_labels):
                 raise ValueError('there should be a min and max value in ppm for each axis')
 
             range_list = []
@@ -101,7 +101,7 @@ class roc:
                 range_list.append([roi_list.pop(0), roi_list.pop(0)])
 
             self.roi = ROI(
-                axis_list=axis_list,
+                axis_labels=axis_labels,
                 range_list=range_list,
             )
 
@@ -163,18 +163,18 @@ class roc:
 
     def _validate_input(self):
         # sometimes proton is labeled as 1H or HN, so standardize on the more common HN
-        if '1H' in self.recovered_peaks.label_list() and 'HN' in self.injected_peaks.label_list():
-            for axis, label in self.recovered_peaks.axis_label_list():
+        if '1H' in self.recovered_peaks.axis_labels() and 'HN' in self.injected_peaks.axis_labels():
+            for axis, label in self.recovered_peaks.axis_keys_labels():
                 if label == '1H':
                     self.recovered_peaks.axis[axis].label = 'HN'
 
-        if set(self.injected_peaks.label_list()) != set(self.recovered_peaks.label_list()):
+        if self.injected_peaks.axis_labels() != self.recovered_peaks.axis_labels():
             raise ValueError('The injected and recovered peak tables have different axis labels.')
 
         if hasattr(self, 'roi'):
-            if len(self.injected_peaks.axis_list()) != len(self.roi.axis_list()):
+            if self.injected_peaks.axis_labels() != self.roi.axis_labels():
                 raise ValueError(
-                    'Number of dimensions for injected and recovered peak lists are not in agreement with ROI list')
+                    'The ROI definition does not have the same axis labels as the peak injected and recovered peak tables.')
 
         if self.injected_peaks.num_peaks() == 0:
             raise ValueError('No recorded peaks in injected peak table')
@@ -310,7 +310,7 @@ class roc:
             n, bins, patches = ax.hist([p[dim] for p in peak_width_all], bins=20)
             ax.set_xlim(.9*np.min(bins), 1.1*np.max(bins))
             ax.set_ylim(-0.1*np.max(n), 1.1*np.max(n))
-            ax.set_title(self.recovered_peaks.label_list()[dim])
+            ax.set_title(self.recovered_peaks.axis_labels()[dim])
 
             # row 1: peak widths for KEEPERS on dimension
             ax = axs[1, dim]
@@ -364,7 +364,7 @@ class roc:
     def plot_peaks(self, dir_out=os.getcwd(), file_basename='peaks', show_figure=False):
 
         # TODO: this axis and dimension handling is very confusing
-        num_dim = len(self.injected_peaks.axis_list())
+        num_dim = len(self.injected_peaks.axis_keys())
         temp_slice_axes = combinations(list(range(1, num_dim + 1)), 2)
         slice_axes = [i for i in temp_slice_axes]
 
@@ -385,8 +385,8 @@ class roc:
 
             fig, ax = plt.subplots()
             fig.subplots_adjust(right=0.75)
-            plt.xlabel(f"{self.recovered_peaks.label_list()[xdim]} Domain [ppm]")
-            plt.ylabel(f"{self.recovered_peaks.label_list()[ydim]} Domain [ppm]")
+            plt.xlabel(f"{self.recovered_peaks.axis_labels()[xdim]} Domain [ppm]")
+            plt.ylabel(f"{self.recovered_peaks.axis_labels()[ydim]} Domain [ppm]")
             plt.scatter(trueRecov_xdim_ppm, trueRecov_ydim_ppm,
                         c='#C5C9C7', marker='s', label='True Recovered')
             plt.scatter(falseRecov_xdim_ppm, falseRecov_ydim_ppm,
