@@ -2,6 +2,7 @@ from dataclasses import dataclass
 from collections import defaultdict
 import numpy as np
 
+
 class PeakOutsideOfMaskError(Exception):
     # use this when the position of a Peak is outside of the ppm ranges covered by a Mask
     pass
@@ -69,9 +70,15 @@ class AxisData:
         """dictionary of axis names where the values are dictionaries of Data objects"""
         self.data = defaultdict(dict)
 
-    def axis_list(self):
+    def axis_keys(self):
         return list(self.data.keys())
 
+    def axis_labels(self):
+        try:
+            return self.get_field(name='label')
+        except KeyError:
+            raise KeyError('not all axes include "label" value')
+
     def set(self, axis, name, value, unit=None):
         axis = axis.upper()
         self.data[axis][name] = Data(name=name, value=value, unit=unit)
@@ -79,7 +86,7 @@ class AxisData:
     def set_data(self, data):
         if not isinstance(data, AxisData):
             raise TypeError('input must be of type AxisData')
-        for ax in data.axis_list():
+        for ax in data.axis_keys():
             self.data[ax] = {**self.data[ax], **data.data[ax]}
 
     def get(self, name, axis=None):
@@ -89,7 +96,7 @@ class AxisData:
         axis: name of axis (str) or list of axis names
         """
         if axis is None:
-            ax_list = self.axis_list()
+            ax_list = self.axis_keys()
         else:
             if not isinstance(axis, list):
                 axis = [axis]
@@ -108,67 +115,53 @@ class AxisData:
 
     def get_field(self, name, axis=None, field='value'):
         # get the AxisData object containing just the desired "name"
-        # extract out just the .value data and return that as a list
+        # extract out just the desired field and return values in a list
         data_out = self.get(axis=axis, name=name)
-        return [data_out.data[ax][name].get(field) for ax in data_out.axis_list()]
+        try:
+            return [data_out.data[ax][name].get(field) for ax in data_out.axis_keys()]
+        except KeyError as e:
+            raise KeyError(e)
 
     def print(self):
-        for axis in self.axis_list():
+        for axis in self.axis_keys():
             print(f"axis = {axis}")
             [d.print() for d in self.data[axis].values()]
 
 
-class ROI:
-    def __init__(self, axis_list, range_list):
+class ROI(AxisData):
+    def __init__(self, key_label_range):
         """
         Define a region of interest (ROI) by giving the min/max values along each axis
-        :param axis_list: axis names (e.g. use 'X', 'Y', 'Z' as used by PeakTable)
+        :param axis_labels: as used by PeakTable (e.g. 'HN' ,'N15', etc.)
         :param range_list: list or tuple containing a min/max list or tuple for each axis
         """
-        # if there is only one axis, then convert singleton inputs into lists
-        if not isinstance(axis_list, (list, tuple)):
-            axis_list = [axis_list]
-        if not isinstance(range_list[0], (list, tuple)):
-            range_list = [range_list]
-        if not all(len(r) == 2 for r in range_list):
-            raise ValueError('must enter a min/max ppm value for each axis as a list of lists')
+        super().__init__()
 
-        if len(axis_list) != len(range_list):
-            raise ValueError('must enter a min/max ppm value for each axis')
+        msg = 'define ROI by providing a list of lists: [key, label, ppm1, ppm2]'
 
-        # dictionary to contain range values for each axis
-        # it is best to access this data through self.range() which can allow no inputs (i.e. get the entire _range
-        # dict) or take an input to specify a single axis (i.e. get the Range object for that axis)
-        self._range = dict()
-        for axis, vals in zip(axis_list, range_list):
-            if not isinstance(vals, (list, tuple)):
-                raise TypeError(f'must provide the min/max values as a list or tuple for axis: {axis}')
-            if len(vals) != 2:
-                raise ValueError(f'must provide min and max as a tuple or list of 2 values for axis: {axis}')
+        if not is_iterable(key_label_range):
+            raise ValueError(msg)
+
+        for info in key_label_range:
             try:
-                # make sure the "min" field gets the minimum value and "max" field gets maximum value
-                # (regardless of order given).   NMR people are strange in how they orient these values
-                self._range[axis] = Range(
-                    min=float(min(vals)),
-                    max=float(max(vals)),
+                key, label, ppm1, ppm2 = info
+                ppm_range = Range(
+                    min=min(float(ppm1), float(ppm2)),
+                    max=max(float(ppm1), float(ppm2)),
                 )
             except ValueError:
-                raise ValueError(f'could not convert ROI range values into floats: {range_list}')
+                raise ValueError(msg)
 
-    def axis_list(self):
-        return self._range.keys()
-
-    def range(self, axis=None):
-        if axis is None:
-            # default => return entire _range dictionary
-            return self._range
-        else:
-            try:
-                return self._range[axis]
-            except KeyError:
-                raise KeyError(f'axis not found: {axis}')
-            except TypeError:
-                raise TypeError('axis should be a string')
+            self.set(
+                axis=key,
+                name='label',
+                value=label,
+            )
+            self.set(
+                axis=key,
+                name='roi',
+                value=ppm_range,
+            )
 
 
 def weightedL2(a, b, w):
@@ -238,4 +231,11 @@ def pairwise_weighted_norm2(A, B, w=None):
         Q = B - a
         N[indexA] = np.sqrt((w * np.matmul(Q, Q.T)).sum())
 
-    return N
\ No newline at end of file
+    return N
+
+def is_iterable(obj):
+    try:
+        iterator = iter(obj)
+        return True
+    except TypeError:
+        return False
